/*******************************************************************************
Copyright (c) 2000-2006 Oracle Corporation

File        : form1013.js

Open Issues : 

$revision_history$
02-mar-2007   Steven Davelaar
  1.9         Added safety net to prevent endless loop when table row count wrongly set
28-feb-2007   Steven Davelaar
  1.8         Fixed error in addChanged Field, "i" changed into "1"
06-feb-2007   Steven Davelaar
  1.7         Skip check for required row items when delete checkbox on new row
              is checked, add rowCount to JhsTable since index number in item names
              is not sequential, added changedFields array so we can detect changed
              fields with autoSubmt = true. 
21-dec-2006   Jaco Verheul
  1.6         Check for required table items now supports inline detail tables
27-nov-2007   Jaco Verheul
  1.5         Fixed bug in checkForChanges. When last item in table was rendered readonly,
              checkForChanges stopped checking next row.
26-sep-2006   Steven Davelaar
  1.4         Added var hasPendingChanges (
22-sep-2006   Steven Davelaar
  1.3         Added check for hasPendingChanges
17-aug-2006   Jaco Verheul
  1.2         Check for required table items improved to handle required el expression.
23-jun-2006   Steven Davelaar
  1.1         Removed methods no longer used in 10.1.3
06-aug-2005   Steven Davelaar
  1.0         Copied from form.js of JHeadstart 10.1.2
******************************************************************************/

/* Debug procedure */
var debugMode = false;

// View type.
var viewType="ADF Faces";

/**
 * debug gives debug messages based on the setting of debugMode
 * the debugMode can be switched of during debugging
 */
function debug(text)
{
  if (debugMode)
  {
    debugMode = confirm(text);
  }
} // debug
// Global variables.
/**
 * form contains the name of the form that should be submitted.
 */
var form;
/**
 * field contains the name of the field that should get some value.
 */
var field;
/**
 * reset contains an array of fields that should be reset before a form
 * submission.
 */
var reset;
/**
 * if ignoreChanges has a value then the appropriate functions don't check
 * for changes.
 */
var ignoreChanges;
/**
 * ignoreChangedFields is an array that contains form
 * fields that are ignored while checking a page for changes.
 */
var ignoreChangedFields = new Array();
/**
 * requiredRowItems contains the names of the items that are required in a table.
 * Required table items cannot be enforced by ADF Faces validation if empty rows are used.
 * The items in this array are of type JhsItem.
 */
var requiredRowItems = new Array();
/**
 * tablesToBeValidated contains all the table names
 * that need to be validated on submit. 
 */
var tablesToBeValidated = new Array();
/**
 * changedItems contains items that are changed, this array is populated for
 * items that have autoSubmit = true so we can still detect the changes
 */
var changedFields = new Array();
/**
 * These are public variables that are used by CheckForChanges, and its helper functions
 */
var dataForm;

/**
 * hasPendingChanges is set in generated pages
 */
var hasPendingChanges;

/**
 * getItemName gets the 'technical' name of an item. In case of ADF Faces table item names have
 * the format [tableName]:[rownum]:[itemName], in which case the rownum needs to be stripped
 * @param item: item to get the name for
 * @return: the name of the item
 */
function getItemName(item)
{
  var type;
  var itemName;
  var colon1;
  var colon2;

  debug("in getItemName");
  if (!item)
  {
    alert("undefined item >"+item+"< in getItemName");
    return null;
  }
  else
  {
    type = getItemType(item);
    if (type == "radiogroup")
    {
      itemName = item[0].name;
    }
    else if (!item.name && item[0] && item[0].name)
    {
      // Apparently item is an array. Just take the name of the first item
      itemName = item[0].name;
    }     
    else
    {
      itemName = item.name;
    }
    // only in case of ADF Faces does the item name contain the rownum
    if (viewType == "ADF Faces")
    {
      itemName = stripRownumFromName(itemName);
    }
  }
  return itemName;
} // getItemName
/**
 * constructTableItemName construct the HTML item name of an item in a table. In case of ADF Faces
 * table item names have the format [tableName]:[rownum]:[itemName]. 
 * @param itemName: the item name including the ADF Faces table name, when applicable
 * @param rownum: the index of the item in the table
 * @return: the name of the item as it will be rendered in the HTML table
 */
function constructTableItemName(itemName, rownum)
{
  var tableItemName;

  if (!itemName)
  {
    alert("undefined itemName in constructTableItemName");
    return null;
  }
  if (!rownum)
  {
    alert("undefined rownum in constructTableItemName");
    return null;
  }

  if (viewType == "ADF Faces")
  {
    tableItemName = rownum+":"+itemName;
  }
  else
  {
    tableItemName = itemName+"["+rownum+"]";
  }
  return tableItemName;
} // constructTableItemName
/**
 * stripTableItemName strips the rowNum from items that have the format
 * [tableName]:[rownum]:[itemName]
 * note: ADF Faces
 * @param itemName: the itemName before stripping the rownum
 * @return: the 'short' name of the item
 */
function stripRownumFromName(itemName)
{
  colon = itemName.lastIndexOf(":");
  if (colon > -1)
  {
    itemName = itemName.substring(colon+1);
  }  
  return itemName;
} // stripRownumFromTableItemName
/**
 * hasChanged checks if the value of item has changed. Thereby also hidden items are checked.
 * @param item: the item to check
 * @return: true when the item has changed, false otherwise
 */
function hasChanged(item)
{
  // debug("In hasChanged for item: "+item);
  var changed = false;

  if (!item)
  {
    alert("undefined item >"+item+"< in hasChanged");
    changed = null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    var type = getItemType(item);
    var itemName = getItemName(item);

    if (!ignoreField(itemName))
    {
      var value;
      var defaultValue;
      var checked;
      var defaultChecked;
      // Check if item is an array. If so, handle differently than if it's
      // a single object
      if (!item.name && item[0] && item[0].name)
      {
        defaultValue = item[0].defaultValue;
        value = item[0].value;
        checked = item[0].checked;
        defaultChecked = item[0].defaultChecked;
      }
      else
      {
        defaultValue = item.defaultValue;
        value = item.value;
        checked = item.checked;
        defaultChecked = item.defaultChecked;
      }    
    
      if (  (  (type == "password" || type == "text" || type == "textarea")
            && defaultValue != value
            )
         || (  (type == "checkbox" || type == "radio")
            && defaultChecked != checked
            )
         )
      {
        changed = true;
      }
      else if (type == "radiogroup")
      {
        for (var i = 0; i < item.length && !changed; i++)
        {
          changed = (item[i].defaultChecked != item[i].checked);
        }
      }
      else if (type == "select-one" || type == "select-multiple")
      {
        var options         = item.options;
        var defaultSelected = 0;
        for (var j = 0; j < options.length; j++)
        {
          if (options[j].defaultSelected)
          {
            defaultSelected = j;
          }
        }

        if (  (defaultSelected != options.selectedIndex)
           && (options.selectedIndex != -1)
           )
        {
          changed = true;
        }
      }
      else if (type == "hidden")
      {
        changed = (item.value != item.defaultValue);
      }
    }
  }

  return changed;
} // hasChanged
/**
 * isEmptyDisplayItem checks if the item is "empty" from the end user perspective.
 * @param item: the item to check
 * @return: true when it is an empty display item, false otherwise
 */
function isEmptyDisplayItem(item)
{
  var empty = true;
  var type;

  if (!item)
  {
    alert("undefined item >"+item+"< isEmptyDisplayItem");
    return null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    var type  = getItemType(item);
    if (type != "hidden")
    {
      empty = isEmptyItem(item);
    }
    else // hidden items are never an empty display item
    {
      empty = false;
    }
    debug("item "+item.name+": empty is " + empty);
    return empty;
  }
} // isEmptyDisplayItem

/**
 * userHasChanged checks if an item has been changed by the user. Therefore only non-hidden
 * items are checked.
 * @param item: the item to check
 * @return: true when the value has changed, false otherwise
 */
function userHasChanged(item)
{
  if (!item)
  {
    alert("undefined item >"+item+"< in userHasChanged");
    return null;
  }
  else
  {
    var type = getItemType(item);
    // only displayed items can return true
    if (type != "hidden")
    {
      return hasChanged(item);
    }
    else
    {
      return false;
    }
  }
} // userHasChanged


/**
 * hasInsertStatus gets the value of the "inserting" field of a specific row in the table
 * Helper function for checkForChanges.
 * note: ADF Faces
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @return: true when inserting equals Y, false otherwise
 */
function hasInsertStatus(tableName, rownum)
{
  debug("dataForm: "+dataForm);
  if (dataForm[tableName+":"+rownum+":IsNewRow"])
  {
    debug("Row "+rownum+" inserting = "+dataForm[tableName+":"+rownum+":IsNewRow"].value);
    return (dataForm[tableName+":"+rownum+":IsNewRow"].value == 'true');
  }
  else
  {
    debug("Row "+rownum+" is existing row");
    return false;
  }
} // hasInsertStatus


/**
 * createTableItemNamesArray creates an array of table item names
 * @param formName: the name of the form
 * @param tableName: the name of the table for which the array should be created
 * Note: for inline detail tables: the tablename is prefixed
 * with parent table name and parent row. For example when 
 * you have a departments table with employees inline, the
 * name of the employees detail table is departments:<rownum parent>:employees
 * @return: an array of table item names ("table name:item name")
 */
function createTableItemNamesArray(formName, tableName)
{
  var form               = document.forms[formName];
  var formElementsCount  = form.elements.length / 1;
  var tempTableItemNames = new Array(formElementsCount);
  var itemFound          = new Array();
  var itemCount          = 0;

  var item;
  var itemName;
  var colon1;
  var colon2;

  // loop through all items in the form and add those items to tempTableItems
  // that belong to tableName
  for (var i = 0 ; i < formElementsCount ; i++)
  {
    item     = form.elements[i];
    itemName = new String(item.name);
    // search backwards starting from end of itemname 
    colon2   = itemName.lastIndexOf(":");
    colon1   = itemName.lastIndexOf(":", colon2 - 1);
    // add item to the array when it is in table with tableName
    if (  colon1 > -1 && colon2 > -1 && colon1 != colon2
       && tableName == itemName.substring(0, colon1)
       )
    {
      // itemFound indicates if a specific item name already has been found
      strippedItemName = stripRownumFromName(itemName);
      if (!itemFound[strippedItemName])
      {
        itemFound[strippedItemName] = true;
        tempTableItemNames[itemCount] = strippedItemName;
        itemCount++;
      }
    }
  }

  // As there may be items in the form than do not belong to the table, the length
  // of tempTableItems may be longer than necessary. Therefore an array is created
  // with a length equal to the number of table items, to be returned.
  var tableItemNames = new Array(itemCount);
  for (var i = 0; i < itemCount; i++)
  {
    tableItemNames[i] = tempTableItemNames[i];
  }

  //  debugString = "Found the following items for table "+tableName+": \n\n";
  //  for (var i = 0 ; i < itemCount; i++) debugString +="- "+tableItemNames[i]+"\n";
  //  debug(debugString);

  return tableItemNames;

} // createTableItemNamesArray

  /**
    Find the name of the table an item is in.
    @param searchItem the name of the item
    @results the name of the table the item is in. When
    the item is in a nested table (table overflow inline), the
    tablename returned contains parents tablename and parent row number also.
    Return empty string when item is not in a table.
  */
  function findTableNameForItem(formName,searchItem) {
    var form               = document.forms[formName];
    var formElementsCount  = form.elements.length / 1;
    
    debug ("in findTableNameForItem");
    for (var i = 0 ; i < formElementsCount ; i++)
    {
      item     = form.elements[i];
      itemName = new String(item.name);
      colon1   = itemName.lastIndexOf(":" + searchItem);
      colon1   = itemName.lastIndexOf(":", colon1-1);
      if (  colon1 > -1) {
        return itemName.substring(0,colon1);
      }
    }
    return null;
  }

/**
 Construct an array with distinct tablesname based on 
 the information in the required items table. The required items
 table is filled in the addRequiredRowItems functions. Calls to 
 this method are generated in the generated pages.
 */
function constructTablesToBeValidated(formName) {
    // loop through all the required items
  for (var i = 0; i < requiredRowItems.length; i++)
  {
    item=requiredRowItems[i];
    
    tableName=findTableNameForItem(formName, item.naam);
    if (tableName) { 
      addTableToBeValidated (formName, tableName);
    }
  }
}

function checkForEmptyRequiredRowItems(formName, rownum, tableName)
{
  var form = document.forms[formName];
  var formItem;
  var type;
  var formItemName;
  var formTableItemName;
  var item = new JhsItem();

  debug("In checkForEmptyRequiredRowItems: "+tableName+" row: "+rownum);
  // skip the row when delete checkbox is checked!
//  formItem = form[tableName+":"+rownum+":"+tableName+"DeleteRow"];
  formItem = form[tableName+":"+rownum+":deleteRow"];
  if (formItem && formItem.checked)
  {
    return true;
  }
  // only check when requiredRowItems have been specified
  if (requiredRowItems)
  {
    // loop through all the required items
    for (var i = 0; i < requiredRowItems.length; i++)
    {
      item=requiredRowItems[i];
      if (item)
      {
        // convert rownum to string
        formTableItemName = tableName+":"+rownum+":"+item.naam;
        debug("checking item name "+formTableItemName);
        formItem = form[formTableItemName];
        debug("checking item "+formItem);
        // non-updateble items are rendered as text, so it should be checked if the item
        // exists before checking the requiredness
        if (formItem)
        {
          debug("Checking for requiredness of item "+i+": "+formTableItemName+", formitem="+formItem);
          if (isEmptyDisplayItem(formItem))
          {
            // Show the message to fill in this field
            alert(getMessage("requiredFields")+item.prompt);
            type = getItemType(formItem);
            // Give the offending item the focus
            if (type == "radiogroup")
            {
              formItem[0].focus();
            }
            else
            {
              formItem.focus();
            }
            // return false as soon as an empty required item has been found
            debug("returning false in checkForEmptyRequiredRowItems because of item "+formItemName+":"+rownum);
            return false;
          }
        }
      }  
    }
    // return true when no empty required items could be found
    return true;
  }
  else
  {
    // return true when there are no requiredRowItems
    return true;
  }
} // checkForEmptyRequiredRowItems

/**
 * deleteRowChecked checks if the delete checkbox has been checked for a row
 * Helper function for checkForChanges.
 * @param tableName: the name of the table
 * @param rownum: the number of the row in the table
 * @return: true when deleteRow has been checked, false otherwise
 */
function deleteRowChecked(tableName, rownum)
{
  debug("In deleteRowChecked, tableName="+tableName+", rownum="+rownum);

  var deleteRowItem = dataForm[tableName+":"+rownum+":deleteRow"];

  if (  deleteRowItem
     && deleteRowItem.checked
     )
  {
    return true;
  }
  else
  {
    // return false for when there is no item deleteRowItem or when it is not checked
    return false;
  }
} // deleteRowChecked

/**
 * Validates form with name formName. 
 * Currently, this entails checking all tables in the form. Make
 * sure that you have registered the tables by calling addTableToBeValidated
 * for each table in the form. 
 */
function validateForm(formName)
{
  if (isFormValidationDisabled())
  {
    debug("form validation is disabled, do NOT perform checkForChanges ");
    enableFormValidation();
    return;
  }
  
  constructTablesToBeValidated(formName);
  
  debug("Number of tables to be validated: "+tablesToBeValidated.length);
  for (var i = 0; i < tablesToBeValidated.length; i ++)
  {
    // Only check the tables in form formName
    if (tablesToBeValidated[i].formName == formName)
    {
      debug("Table to be validated: "+tablesToBeValidated[i].tableName);
      // Check whether the form and the table exist before
      // validating it. It might be not the case when autoQuery
      // is turned off and no search criteria have been entered yet. 
      var dataForm = document.forms[formName];    
//      if (dataForm && dataForm[tablesToBeValidated[i].tableName+":length"])
      if (dataForm)
      {
        var b = checkForChanges(formName, tablesToBeValidated[i].tableName);
        if (!b) return false;
      }
    }
  }
  clearChangedFields();
  return true;
}

/**
 * This function should never be called directly anymore. Use validateForm
 * instead.
 *
 * checkForChanges is used when a table is submitted. The table should contain
 * a field 'hasChanges' for each row. This function populates those fields. If
 * a row has not changed then the value is 'N', if it has changes the value is
 * 'Y'. It also calls checkRequiredRowItems() to validate if there are required
 * items without a value.
 * @param formName: the name of the form
 * @param tableName: the name of the table to check for changes
 * @return: false when an error occured regarding a required item, true otherwise
 */
function checkForChanges(formName, tableName)
{
  debug("in checkForChanges for table "+tableName);
  // Set dataForm variable so that we don't need to supply the form name
  // to all helper functions of this method.
  dataForm       = document.forms[formName];

  // Create array of all table items.
  var tableItems = createTableItemNamesArray(formName, tableName);
  var changeMade = false;
  var initialStatus;
  var item;

  // handle empty array to make code more robust.
  // otherwise next while loop will hang.
  if (tableItems.length == 0) 
  {
    debug ("no items found in table");
    return true;
  }

// Loop through all the table rows
// Note that the number of table rows is set on JhsTable object.
// However the table item names do not necessarily have names like
// tableName:index:itemName where index is increasing with 1 starting from 1.
// If a search has been performed and some new rows are shown compared to
// previoues page rendering, the index number used within the item name can
// contain gaps, for example:
// DepartmentTable:1:DepartmentId,DepartmentTable:10:DepartmentId,DepartmentTable:11:DepartmentId
// so we must increase the index number until we find the next row items, and maintain
// a separate counter to check whether we have processed all table rows
  // find the JhsTable element of current table, and get the row count
  var rowCount;
  for (i = 0; i<tablesToBeValidated.length ; i++) 
  {
    if (tablesToBeValidated[i].formName==formName &&
        tablesToBeValidated[i].tableName==tableName) 
    {
      tablePresent=true;
      rowCount = tablesToBeValidated[i].rowCount
    }
  }
  if (!tablePresent)
  {
    // do nothing!
    return;
  }
  // i is counter to get item names!
  i=-1
  for (r = 0; r<rowCount ; r++) 
  {
    // first determine the index number used in item name for this row
    i++;
    item = dataForm[tableName+":"+i+":"+tableItems[0]];
    // to prevent an endless loop in case something has gone wriong with setting
    // the row count, we stop at 50.
    while (!item && i < 50)
    {
      i++
      item = dataForm[tableName+":"+i+":"+tableItems[0]];
    }
    
    if (!item)
    {
      return true;
    }
    
    changeMade    = false;
    if (deleteRowChecked(tableName, i))
    {
      // skip this row
      continue;
    }

    if (hasInsertStatus(tableName,i))
    {
      // Check whether the user made changes to this new row!
      changeMade = false;
      // Loop through all items in the row to see if one was changed
      for (var j = 0 ; j < tableItems.length && !changeMade; j++)
      {
        debug("current item: "+tableItems[j]);
        // set hasChanges to 'Y' for row as soon as a changed item has been found
        itemName = tableName+":"+i+":"+tableItems[j];
        item = dataForm[itemName];
        // non-updateble items are rendered as text, so it should be checked if the item
        // exists before calling userHasChanged
        if (item && userHasChanged(item))
        {
          debug("Field "+tableItems[j]+" on row "+i+" was changed");
          changeMade = true;
        }
        else
        {
          // check whether item is in changedFields array
         for (var k = 0 ; k < changedFields.length && !changeMade; k++)
         {
           if (changedFields[k]==itemName)
           {
             changeMade = true;
             debug("Field "+changedFields[k]+" on row "+i+" was changed");
           }
         }
        }
      }
    }
    else
    {
      // Existing record, always check
      changeMade = true;
    }
    debug("change made in row "+i+": "+changeMade);
    // If a change was made to this row 
    // check if the required fields for this row are filled in. If not, abort the loop
    // and return false;
    if (  changeMade
       && !checkForEmptyRequiredRowItems(formName, i, tableName)
       )
    {
      debug("checkForChanges returns false for table "+tableName);
      return false;
    }
  }
  debug("checkForChanges returns true for table "+tableName);
  return true;
} // checkForChanges


/**
 * getItemType returns the type of an item. If it is a radio group with more than 1 option,
 * the item is itself an array of radio group items.
 * @param item: the item for which to return the type
 * @return: the type of the item
 */
function getItemType(item)
{
  var type = null;

  if (!item)
  {
    alert("undefined item >"+item+"< in getItemType");
  }
  else
  {
    if (  !item.type
       && !isString(item)
       && item[0].type == "radio"
       ) // the item is an array of radio items
    {
      type = "radiogroup";
    }
    else if (!item.type && item[0] && item[0].type)
    {
      // Apparently item is an array. Just take the type of the first item
      type = item[0].type;
    }    
    else if (!item.type)
    {
      alert("unrecognized item >"+item+"< in getItemType");
    }
    else
    {
      type = item.type;
    }
  }

  return type;
} // getItemType

/**
 * Constructor of the JhsTable object
 * This object is used for registering all tables to be checked
 */
function JhsTable(formName, tableName, rowCount) {
  this.formName  = formName;
  this.tableName = tableName;
  this.rowCount = rowCount;
}

/**
 * Constructor of JhsItem object.
 * This object is used for registering an array with required table items.
 */

function JhsItem(naam, prompt) {
  this.naam = naam;
  this.prompt = prompt;
}

/**
 * isEmptyItem checks if an item is null or has an empty value
 * note: ADF Faces & JSP
 * @param item: the item to check
 * @return: true when the item is null or has an empty value
 * (leading and trailing spaces are stripped before checking)
 */
function isEmptyItem(item)
{
  var type;

  if (!item)
  {
    alert("undefined item >"+item+"< in isEmptyItem");
    return null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    type = getItemType(item);

    // an isolated radio button should be ignored, i.e. return false
    if (type == "radio")
    {
      return false;
    }
    // a checkbox should be ignored, i.e. return false, as checked/unchecked are equivalent to
    // something like Y/N
    else if (type == "checkbox")
    {
      return false;
    }
    // the value of a radiogroup depends on one of the items having the property checked
    else if (type == "radiogroup")
    {
      var isEmpty = true;
      for (var i=0; i < item.length && isEmpty; i++)
      {
        isEmpty = !item[i].checked;
      }
      return isEmpty;
    }
    // the value of poplists need to be retrieved in a different way
    else if (type == "select-one" || type == "select-multiple")
    // the value of poplists is depends on the selectedIndex
    {
      // strip all leading and trailing spaces (in Netscape hidden items contain spaces)
      if (item.selectedIndex==-1)
      {  
        return '';
      }      
      return (stripSpaces(item.options[item.selectedIndex].value).length == 0);
    }
    // in other cases (password, text, textarea) the value depends on the property value
    else if (item.value)
    {
      // strip all leading and trailing spaces (in Netscape hidden items contain spaces)
      return (stripSpaces(item.value).length == 0);
    }
    else // when item has no value, it is considered to be empty
    {
      return true;
    }
  }
} // isEmptyItem


/**
 * resetValues loops through the global array 'reset' and resets all the fields
 * that correspond to the array items.
 * @param formName: the name of the form
 */
function resetValues(formName)
{
  var form = document.forms[formName];
  var item;
  var itemType;
  if (reset)
  {
    for (var i = 0 ; i < reset.length ; i++)
    {
      item = form[reset[i]];
      if (item)
      {
        itemType = getItemType(item);
        if (  itemType != "radio"
           && itemType != "radiogroup"
           && itemType != "checkbox"
           && itemType != "select-one"
           )
        {
          item.value = "";
        }
      }
    }
  }
} // resetValues


/**
 * resetFormValues loops through all forms, then through all form items of type 'hidden',
 * and if the hidden item did not have a default value, it is reset. The effect of this
 * is therefore similar to reloading a page, but only for hidden fields. This procedure
 * is called by default from the onLoad event of each ADF Faces page, to make sure that (navigation)
 * hidden fields are cleared when the user performs a navigation, uses the backbutton, and
 * performs another navigation. On iExploder the hidden fields are not cleared automatically.
 * on Netscape they are.
 *       this function does not work properly for Netscape 7.0.2 as item.defaultValue for
 *       hidden items will be the same as the changed value
 */
function resetFormValues()
{
  var form;
  var item;
  for (var f = 0 ; f < document.forms.length ; f++)
  {
    form = document.forms[f];
    for (var i = 0 ; i < form.elements.length ; i++)
    {
      item = form.elements[i];
      if (  item.type == "hidden"
         && item.defaultValue == ""
         )
      {
        item.value = "";
      }
    }
  }
} // resetFormValues



/**
 * isString checks if the argument is a string
 * @param 'argument': the argument to check
 * @return: true if the 1st argument is a string, false otherwise
 */
function isString()
{
  if (arguments[0] == null)
  {
    return false;
  }
  else if (typeof arguments[0] == 'string')
  {
    return true;
  }
  else if (  (typeof arguments[0] == 'object')
          && (arguments[0].constructor)
          )
  {
    var criterion = arguments[0].constructor.toString().match(/string/i);
    return (criterion != null);
  }
  else
  {
    return false;
  }
}


/**
 * stripSpaces removes leading and trailing spaces from a string
 * @param aString: the string to strip
 * @return: the stripped string
 */
function stripSpaces(aString)
{
  if (!isString(aString))
  {
    alert("undefined string >"+aString+"< in stripSpaces");
    return null;
  }
  else
  {
    // strip leading spaces
    while (aString.substring(0,1) == " ")
    {
      aString = aString.substring(1);
    }
    // strip trailing spaces
    while (aString.substring(aString.length-1,aString.length) == " ")
    {
      aString = aString.substring(0,aString.length-1);
    }

    return aString
  }
} // stripSpaces


/**
 * isNumber checks if a string (after stripping leading and trailing spaces) represents
 * a numeric value. Empty strings are not considered to be a number.
 * This validator is more strict than the ADF Faces decimal validator, because it does not
 * accept values like "1a2".
 * Remark: values like "1e3" are accepted, because its interpreted as 1000 (one raised
 * to the power of three)
 * @param aString: the string to check
 * @return: true when aString represents a number, false otherwise
 */
function isNumber(aString)
{
  if (!isString(aString))
  {
    alert("undefined string >"+aString+"< in isNumber");
    return null;
  }
  else
  {
    // empty strings are not numbers
    if (stripSpaces(aString).length == 0)
    {
      return false;
    }

    // if aString evaluates to number zero, return true
    if ((aString*aString) == 0)
    {
      return true;
    }

    // in all other cases, return true if item divided by itself it returns 1
    return (aString/aString == 1);
  }
} // isNumber


/**
 * ignoreField checks if an item should be ignored when checking for changed items.
 * Items to be ignored are expected to start with an underscore (_) or be in the array
 * ignoreChangedFields
 * @param itemName: the item to be checked
 * @return: true when the item should be ignored (when the name start with an
 * underscore or when it is in the array ignoreChangedFields), false otherwise
 */
function ignoreField(itemName)
{
  debug("In ignoreField with itemName: "+itemName);

  itemNameStripped = stripRownumFromName(itemName);

  if (itemNameStripped.substring(0,1) == "_")
  {
    return true;
  }

  if (ignoreChangedFields)
  {
    for (var j = 0 ; j < ignoreChangedFields.length ; j++)
    {
      if (ignoreChangedFields[j] == itemNameStripped)
      {
        return true;
      }
    }
  }
  // now check whether the unstripped item name should be ignored 
  if (ignoreChangedFields)
  {
    for (var j = 0 ; j < ignoreChangedFields.length ; j++)
    {
      if (ignoreChangedFields[j] == itemName)
      {
        return true;
      }
    }
  }

  // return false in all other cases
  return false;

} // ignoreField


/**
 * hasChanges loops over all forms in the document and checks if one or more of the form items
 * have been changed by the user. If there are (unsumitted) changes, the user is asked
 * whether he wants to leave the page, or return to the page and save the changes. By calling
 * ignoreField, some of the items are ignored by hasChanges().
 * @return: true when there are unsubmitted changes (that might be because of an error on
 * the page), false otherwise
 */
function hasChanges()
{
  // first check for middle tier pending changes, this
  // var is set in pageConfig.jspx
  if (hasPendingChanges && hasPendingChanges == 'true' )
  {
    debug("hasChanges returns true because hasPendingChanges has a value");
    return true;
  }
  for (var i = 0 ; i < document.forms.length ; i++)
  {
    var form = document.forms[i];

    debug("checking form >"+form.name+"< for changes");

    // return true when there are errors on the page
    // S. Davelaar 10-02-2006: prefix with dataForm: for JSF !!! 
    if (form.elements["dataForm:pageConfig:pageConfig:hasChanges"])
    {
      if (stripSpaces(form.elements["dataForm:pageConfig:pageConfig:hasChanges"].value) != "" )
      {
        debug("hasChanges returns true because hasChanges has a value");
        return true;
      }
      // else it concerns a select page
    }

    // return true when one of the items in any form have been changed by the user
    for (var k = 0 ; k < form.elements.length ; k++)
    {
      var item = form.elements[k];
      // fix for error in StripRowNum on UIX table pages in JDev 10.1.2
      // a new HTML element without a type is present on the page (for single selection ADF Faces element)
      if (!item.type)
      {
        continue;
      }

      if (  !ignoreField(item.name)
         && userHasChanged(item)
         )
      {
        debug("hasChanges returns true because item >"+item.name+"< has changed");
        return true;
      }
    }
  }

  // in any other case, return false
  debug("hasChanges returns false");
  return false;
} // hasChanges


/**
 * confirmChanges checks if one of the form items has changed by calling hasChanges
 * If so, it asks the user to confirm if he wants to leave the page without saving,
 * or return to the same page. After the user has the opportunity to save the changes.
 * note: ADF Faces & JSP
 * @return; true when there are no changes or when the user choose to leave the page
 * whithout saving them, false otherwise.
 */
function confirmChanges()
{
  var anyFormChanged = hasChanges();

  var confirmed;

  if (anyFormChanged)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!anyFormChanged || confirmed)
  {
    return true;
  }
  else
  {
    return false;
  }
} // confirmChanges


/**
 * This function should be called from a button on a singleSelection
 * table. It check whether there is a selected row and, if so,
 * whether that row is not a new (and therefore not yet inserted) row.
 * To do this, the parameter checkItemName is passed. This should
 * be a field that is always null for new rows (no matter which fields
 * the end user already filled in, and that is never null for queried
 * rows. A perfect candidate would be the readOnly<PrimaryKey> formvalue.
 * To omit this check for new rows (for example if the table does not
 * have any new rows), call checkSelectedRow without the 3rd argument.
 */
function checkSelectedRow(formName, tableName, checkItemName)
{
  var form  = document.forms[formName];
  var field = form[tableName+":selected"];
  var j     = -1;

  if (field.length)
  {
    // than more than one radio group item is selectable
    for (var i = 0; i < field.length; i++)
    {
      debug("Checking if row "+i+" is checked");
      if (field[i].checked)
      {
        j = i;
      }
    }

    if (j == -1)
    {
      alert(getMessage("noRowsSelected"));
      return false;
    }
  }
  else
  {
    if (!field.checked)
    {
      alert(getMessage("noRowsSelected"));
      return false;
    }
  }

  if (checkItemName)
  {
    var checkItem = form[tableName+":"+checkItemName+":"+j];
    if (stripSpaces(checkItem.value) == "")
    {
      alert(getMessage("newRowSelected"));
      return false;
    }
  }

  var anyFormChanged = false;
  if (!ignoreChanges)
  {
    anyFormChanged = hasChanges();
  }

  var confirmed;
  if (  !anyFormChanged
     || confirm(getMessage("hasChanged"))
     )
  {
    return true;
  }
  else
  {
    return false;
  }
} // checkSelectedRow

/**
 * replace replaces all occurences of a specific text in a string
 * @param string: the string for which text needs to be replaced
 * @param text: the text that needs to be replaced
 * @param by: the replacement text
 */
function replace(aString, text, by)
{
  if (  !isString(aString)
     || !isString(text)
     || !isString(by)
     )
  {
    alert("undefined aString >"+aString+", "+text+", "+by+"< in replace");
    return null;
  }
  else
  {
    var strLength = aString.length;
    var txtLength = text.length;

    // return the original string when either the string or the text to be
    // replaced have length 0
    if (  (strLength == 0)
       || (txtLength == 0)
       )
    {
      return aString;
    }

    var i = aString.indexOf(text);
    if (  (!i)
       && (text != aString.substring(0,txtLength))
       )
    {
      return aString;
    }

    if (i == -1)
    {
      return aString;
    }

    var newstr = aString.substring(0,i) + by;
    if ((i + txtLength) < strLength)
    {
      newstr += replace(aString.substring(i+txtLength,strLength),text,by);
    }

    return newstr;
  }
} // replace

/**
 * Loops through all the forms and look for the first occurence of an element with name
 * elementName.
 *
 * @param elementName The name of the element to look for
 * @result The first occurence of the element with name elementName. null, if no
 *         element with name elementName could be found
 */
function getFormElement(elementName)
{
  var result = null;

  for (var i = 0 ; i < document.forms.length ; i++)
  {
    var searchForm = document.forms[i];
    if (searchForm.elements[elementName])
    {
      result = searchForm.elements[elementName];
    }
  }

  return result;
}

/**
 * Loops through all the forms and looks for the first occurence of an element with name
 * elementName. The function returns the value of this element. 
 *
 * note: JSP
 * @param elementName The name of the element to look for
 * @result The value of the first occurence of the element with name elementName. null, if no
 *         element with name elementName could be found
 */
function getFormElementValue(elementName)
{
  var result = null;
  var formElement = getFormElement(elementName);
  if (formElement)
  {
    result = formElement.value;
  } 
  return result;
}

function getDataForm()
{
  return document.forms[dataFormName];
}

 function setClassOnElement( elementId, className)
 {
    el = document.getElementById(elementId);
    agt = navigator.userAgent.toLowerCase();
    if (agt.indexOf('mozilla')!=-1 && agt.indexOf('spoofer')==-1 && agt.indexOf('compatible') == -1)
      // mozilla
      el.setAttribute("class", className);
    else
      el.setAttribute("className", className);
 }


function addToIgnoreChangedFields(fields) 
{
  curlength = ignoreChangedFields.length;
  for (i = 0; i<fields.length; i++)
    ignoreChangedFields[curlength + i] = fields[i];
}

function addChangedField(field) 
{
  curlength = changedFields.length;
  changedFields[curlength + 1] = field;
}

function clearChangedFields() 
{
  changedFields = new Array();
}

function addRequiredRowItems(formName, items) 
{
  debug ("in addRequiredRowItems");
  curlength = requiredRowItems.length;

  for (i = 0; i<items.length; i++) 
  {
    requiredRowItems[curlength + i] = items[i];
  }
}

function addTableToBeValidated(form, tableName)
{
  debug ("addTableToBeValidated " + tableName);
  
  // check whether this combination of form and table already exists in 
  // the array.
  
  tablePresent=false;
  for (i = 0; i<tablesToBeValidated.length ; i++) {
    if (tablesToBeValidated[i].formName==form &&
        tablesToBeValidated[i].tableName==tableName) {
      tablePresent=true;
    }
  }
  
  if (!tablePresent) {
    debug ("add table " + tableName);
    var jhsTable = new JhsTable(form, tableName);
    tablesToBeValidated[tablesToBeValidated.length]=jhsTable;
  }
}

function setTableRowCount(form, tableName, rowCount)
{
  debug ("setTableRowCount " + tableName + "rowcount: "+rowCount);
  var jhsTable = new JhsTable(form, tableName, rowCount);
        
  tablePresent=false;
  for (i = 0; i<tablesToBeValidated.length ; i++) {
    if (tablesToBeValidated[i].formName==form &&
        tablesToBeValidated[i].tableName==tableName) 
    {
      tablePresent=true;
      tablesToBeValidated[i]=jhsTable
    }
  }
  
  if (!tablePresent) {
    debug ("add table " + tableName);
    tablesToBeValidated[tablesToBeValidated.length]=jhsTable;
  }
}


/**
 * Call this function when your input control creates a partial page requests
 * that submits the form, and Jheadstart should NOT do custom javascript
 * validation like checking required items in a table
 */
function disableFormValidation()
{
  if (getDataForm().validateForm)
  {
    getDataForm().validateForm.value='false';
  }
}  

/**
 * Call this function to enable Jheadstart ustom javascript
 * validation like checking required items in a table
 */
function enableFormValidation()
{
  if (getDataForm().validateForm)
  {
    getDataForm().validateForm.value = "true";
  }
}  

function isFormValidationDisabled()
{
  if (getDataForm().validateForm)
  {
    return getDataForm().validateForm.value=='false';
  }
  else
  {
    return false;
  }
}  

function alertForChanges()
{
  var changed = changedFields.length>0 || hasChanges();
  var confirmed;
  if (changed)
  {
     confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    clearChangedFields();
    return true;
  }
  else
  {
    return false;
  }
  
}
